'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var collectMetrics = _interopDefault(require('speedmetrics'));
var puppeteer = require('puppeteer');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var _this = undefined;
var disabledByDefault = function (category) { return "disabled-by-default-" + category; };
var categories = [
    '-*',
    'toplevel',
    'loading',
    'v8.execute',
    'devtools.timeline',
    'devtools.timeline.async',
    disabledByDefault('devtools.timeline.stack'),
    disabledByDefault('devtools.timeline'),
    disabledByDefault('devtools.timeline.frame'),
    'navigation',
    'blink.console',
    'blink.user_timing',
    disabledByDefault('v8.runtime_stats_sampling'),
    disabledByDefault('v8.cpu_profiler'),
    disabledByDefault('v8.cpu_profiler.hires')
];
var startTracing = function (client) { return __awaiter(_this, void 0, void 0, function () {
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4, client.send('Tracing.start', {
                    transferMode: 'ReturnAsStream',
                    categories: categories.join(','),
                    options: 'sampling-frequency=1000',
                    bufferUsageReportingInterval: 500
                })];
            case 1:
                _a.sent();
                return [2];
        }
    });
}); };

var _this$1 = undefined;
var stopTracing = function (client) {
    var readTrace = function (handle) { return __awaiter(_this$1, void 0, void 0, function () {
        var buffer, eof, response;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    buffer = [];
                    eof = false;
                    _a.label = 1;
                case 1:
                    if (!!eof) return [3, 3];
                    return [4, client.send('IO.read', { handle: handle })];
                case 2:
                    response = _a.sent();
                    buffer.push(response.data);
                    eof = response.eof;
                    return [3, 1];
                case 3: return [4, client.send('IO.close', { handle: handle })];
                case 4:
                    _a.sent();
                    return [2, JSON.parse(buffer.join(''))];
            }
        });
    }); };
    return new Promise(function (resolve) {
        client.once('Tracing.tracingComplete', function (event) {
            readTrace(event.stream).then(resolve);
        });
        client.send('Tracing.end', {});
    });
};

var _this$2 = undefined;
var haast = function (urlOrCallback) { return __awaiter(_this$2, void 0, void 0, function () {
    var browser, page, trace, metrics;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4, puppeteer.launch()];
            case 1:
                browser = _a.sent();
                return [4, browser.newPage()];
            case 2:
                page = _a.sent();
                return [4, startTracing(page._client)];
            case 3:
                _a.sent();
                if (!(typeof urlOrCallback === 'string')) return [3, 5];
                return [4, page.goto(urlOrCallback, { waitUntil: 'networkidle0' })];
            case 4:
                _a.sent();
                return [3, 7];
            case 5: return [4, urlOrCallback(page)];
            case 6:
                _a.sent();
                _a.label = 7;
            case 7: return [4, stopTracing(page._client)];
            case 8:
                trace = _a.sent();
                metrics = collectMetrics(trace.traceEvents);
                return [4, browser.close()];
            case 9:
                _a.sent();
                return [2, metrics];
        }
    });
}); };

exports.default = haast;
